@@ -1,9 +1,15 @@
 #! /usr/bin/env python
 
-ver = "0.10.110.18.13"
+ver = "0.11.053.10.70"
 """
 autor Bartlomiej Ceglik
 e-mail yoyoek[no spam]wp.pl
+
+
+BIG THX for:
+
+John
+
 """
 
 
@@ -89,8 +95,10 @@ except:
 
 import sqlite3
 import time
+import re
 
 lista_of_icals = []
+possible_calendar_names = dict()
 sql_file = '/home/user/.calendar/calendardb'
 sql_debug = 0
 error = 0
@@ -109,6 +117,14 @@ def Read_ical_from_web( host, file ):
         return ''
 
     r1 = conn.getresponse()
+    try:
+        possible_name = r1.getheader('content-disposition')
+        guessname_re = re.compile('filename=(?:.*/)?(.+?)(?:\.ics)?(?:"|;|$)', flags=re.IGNORECASE)
+        possible_name = guessname_re.search(possible_name)
+        if possible_name <> None:
+            possible_calendar_names[file] = possible_name.group(1)
+    except:
+        print "Can't guess a calendar name for %s" % file
     if r1.reason <> 'OK':
         print "connection error result\n-----------------------"
         print r1.status
@@ -173,7 +189,7 @@ def Sql_insert_correct(s_con, s_id, s_into, s_values, s_vars ):
             c.execute(q,s)
     except:
         if len( s_vars ) == len( s_values ):
-            print "[q] insert get s_var as singiel record"
+            #print "[q] insert get s_var as singiel record"
             #print "\n-------------\n"+str(s_vars)+"\n-singiel------------"
             c.execute(q,s_vars)
         else:
@@ -184,7 +200,7 @@ def Sql_insert_correct(s_con, s_id, s_into, s_values, s_vars ):
     if s_id<>'':
         return Sql_query_select_one_value( s_con, s_id, s_into, '1 ORDER BY '+s_id+' DESC' )
     else:
-        return ''
+        return c.lastrowid
 
 def Sql_delete(s_con, s_from, s_where ):
     c = s_con.cursor()
@@ -303,7 +319,10 @@ def Run_import( dialog = '' ):
                 cal_name = cal_name_item[0]['X-WR-CALNAME']+" (webcal)"
             except:
                 print "[Cal name not deff :/ set from X-WR-CALDESC]"
-                cal_name = cal_name_item[0]['X-WR-CALDESC']+" (webcal)"
+		try:
+	                cal_name = cal_name_item[0]['X-WR-CALDESC']+" (webcal)"
+		except:
+			cal_name = possible_calendar_names[ical[1]]
 
             print "[Start ical iter]    [ical name] ["+cal_name+"]"
 
@@ -331,37 +350,49 @@ def Run_import( dialog = '' ):
 
             print "[Start ical iter]    [db insert some events :)]"
             count_total = len( cal.walk('vevent') )
-            ev_ta = []
             for e in cal.walk('vevent'):
-                tStart = ParseStrTime_to_unix(e['dtstart'])+(ical[2]*3600)
-                tEnd = ParseStrTime_to_unix(e['dtend'])+(ical[2]*3600)
+		try:
+	                tStart = ParseStrTime_to_unix(e['dtstart'])+(ical[2]*3600)
+        	        tEnd = ParseStrTime_to_unix(e['dtend'])+(ical[2]*3600)
 
-                location = Call_featch_val(e,'location')
-                desc = Call_featch_val(e, 'description')
-                entryDate = Call_featch_val(e, 'created')
-                if entryDate:
-                    entryDate = ParseStrTime_to_unix(entryDate)
-                    if entryDate < 0:
-                        entryDate = 0
-                modDate = Call_featch_val(e, 'last-modified')
-                if modDate:
-                    modDate = ParseStrTime_to_unix(modDate)
-                    if modDate < 0:
-                        modeDate = 0
+	                location = Call_featch_val(e,'location')
+        	        desc = Call_featch_val(e, 'description')
+                	entryDate = Call_featch_val(e, 'created')
+	                if entryDate:
+        	            entryDate = ParseStrTime_to_unix(entryDate)
+                	    if entryDate < 0:
+                        	entryDate = 0
+	                modDate = Call_featch_val(e, 'last-modified')
+        	        if modDate:
+                	    modDate = ParseStrTime_to_unix(modDate)
+	                    if modDate < 0:
+        	                modeDate = 0
+	                flags = -1
+	                is_all_day = 0
+	                if e.has_key('RRULE'):
+		                  flags = 3
+	                if e.has_key('X-MICROSOFT-CDO-ALLDAYEVENT') and e['X-MICROSOFT-CDO-ALLDAYEVENT'] == 'TRUE':
+		                  is_all_day = 1
+	                #print "entry(%s) mod(%s)" % ( entryDate, modDate )
+	                last_id = Sql_insert_correct(s_con,
+	                       '',
+	                       'Components',
+	                       ['CalendarId',      'ComponentType',    'Flags','DateStart',    'DateEnd',  'Summary',          'Location',         'Description',      'Status',   'Uid',     'Until', 'AllDay','CreatedTime', 'ModifiedTime', 'Tzid','TzOffset'],
+	                       [cal_id,        '1',                flags,   tStart,         tEnd,        e['summary'],      location,           desc,               '-1',       '0',        '-1',   is_all_day,    str(entryDate),     str(modDate),        ':UTC', '0']
+	                       )
+	                if e.has_key('RRULE'):
+		                # Rtype of 6 means don't let the user edit the recursing
+		                Sql_insert_correct(s_con,
+		                      '',
+		                      'RECURSIVE',
+		                      ['Id',      'Rrule',         'Rdates',    'Exdates','Exrule',    'RecurrenceId',  'Rtype'],
+		                      [last_id,   str(e['RRULE']),  '',          '',        '',         '-1',            '6']
+		                      )
 
-                #print "entry(%s) mod(%s)" % ( entryDate, modDate )
+		except Exception, ex :
+			print "Skipping event %s due to %s" % (e['UID'], ex)
 
-                ev_ta.append(
-                             [cal_id,        '1',                '-1',   tStart,         tEnd,        e['summary'],      location,           desc,               '-1',       '0',        '-1',   '0',    str(entryDate),     str(modDate),        ':UTC', '0']
-                             )
-
-            print "[Start ical iter]    [db insert %i/%i] [%s]" % (len(ev_ta),count_total, cal_name)
-            Sql_insert_correct(s_con,
-                        '',
-                        'Components',
-                        ['CalendarId',      'ComponentType',    'Flags','DateStart',    'DateEnd',  'Summary',          'Location',         'Description',      'Status',   'Uid',     'Until', 'AllDay','CreatedTime', 'ModifiedTime', 'Tzid','TzOffset'],
-                        ev_ta
-                        )
+            print "[Start ical iter]    [db insert %i] [%s]" % (count_total, cal_name)
 
             print "[Start ical iter]    [sql try disconnect]"
             s_con.close()
